# -*- coding: utf-8 -*-
# __author__ = 'rfooox'
# cid_hash_file function from https://github.com/binux/lixian.xunlei/blob/master/libs/tools.py
# Original Gist: https://gist.github.com/ihciah/30eda05ca36ee9f9f190067538b0ae04
# Github Repo: https://github.com/ihciah/subtitle

import hashlib
import os
import sys
import urllib3
import re
import logging
from urllib3.util import parse_url, Url
from rSubs import config
from rSubs.http_dns import dns_hijack
dns_hijack(http_dns=config.http_dns)


class SubtitleDownloader:
    @staticmethod
    def version():
        return u'VERSION %s.\n\r' \
               u'Subtitle downloader by ihciah >_< \n\r' \
               u'https://github.com/ihciah/subtitle' % config.__version__

    @staticmethod
    def url_get(url):
        url_part = parse_url(url)
        UrlObj = Url(url_part.scheme, url_part.auth, url_part.host, url_part.port,
                     url_part.path or None, url_part.query or None, url_part.fragment or None)
        conn = urllib3.connection_from_url(url)
        response = conn.request('GET', UrlObj.request_uri)
        return response

    @staticmethod
    def download_srt(subtitle_url, video_base_path, video_name, num):
        dot = subtitle_url.rfind(u'.')
        if dot < 0:
            return

        # 确定下载文件类型
        subtitle_type = subtitle_url[dot + 1:].lower()

        # 如果url中类型不匹配则不下载
        if subtitle_type not in config.subtitle_types:
            return

        # 打开字幕文件
        srt_file = os.path.join(video_base_path, video_name + u'.zh.%d.' % num + subtitle_type)
        # 字幕已存在 返回
        if os.path.isfile(srt_file):
            return
        # 下载字幕
        response = SubtitleDownloader.url_get(subtitle_url)

        if response.status == 200:
            # 下载成功将内容写入字幕文件
            with open(os.path.join(video_base_path, video_name + u'.zh.%d.' % num + subtitle_type), 'wb') as f:
                f.write(response.data)

    @staticmethod
    def cid_hash_file(path):
        h = hashlib.sha1()
        size = os.path.getsize(path)
        with open(path, 'rb') as stream:
            if size < 0xF000:
                h.update(stream.read())
            else:
                h.update(stream.read(0x5000))
                stream.seek(size//3)
                h.update(stream.read(0x5000))
                stream.seek(size-0x5000)
                h.update(stream.read(0x5000))
        return h.hexdigest().upper()

    @staticmethod
    def fetch_subtitle_list(cid):
        patten = re.compile(b'surl="(.*?)"')
        url_base = u'http://subtitle.kankan.xunlei.com:8000/submatch/%s/%s/%s.lua'
        r = SubtitleDownloader.url_get(url_base % (cid[:2], cid[-2:], cid)).data  # 拼接url
        # 找页面中前n（download_count）个url
        srt_urls = patten.findall(r)[:config.download_count]
        # 返回utf-8编码的url list
        return list(map(lambda url: url.decode(u'utf-8'), srt_urls))

    @staticmethod
    def download_subtitle(video):
        # 传入参数 video，file or path
        logging.info(u"Processing: {}".format(video))
        if not video:
            # 文件为空退出程序
            logging.error(u"Video file or dir does not exist: %s".format(video))
            return -1
        # 判断是否为path
        if os.path.isdir(video):
            if sys.version_info.major == 2:
                # python 2版本 处理
                code = sys.getfilesystemencoding()  # 获取当前系统的编码格式
                map(lambda path: SubtitleDownloader.download_subtitle(os.path.join(video, path.decode(code))),
                    os.listdir(video))
            else:
                # 递归遍历当前目录的文件夹与文件，若是文件夹则继续递归
                list(map(lambda path: SubtitleDownloader.download_subtitle(os.path.join(video, path)),
                         os.listdir(video)))
            return

        # video不是文件 异常退出程序
        if not os.path.isfile(video):
            logging.error(u"Video file does not exist: {}".format(video))
            return -1


        # 将文件拆分为路径和文件名称
        video_base_path, video_filename = os.path.split(os.path.abspath(video))

        # video中解析不出内容，异常退出
        if not video_base_path or not video_filename:
            logging.error(u"Something error: {}".format(video))
            return -1

        # 查询最右边的. 位数
        dot = video_filename.rfind(u'.')
        if dot <= 0:
            # .位置小于等于0无法甄别是否为视频文件，异常退出
            logging.info(u"File doesn't have a suffix or without a name: {}".format(video_filename))
            return -1

        # 将文件名称与文件类型拆分开
        video_name = video_filename[:dot]
        video_type = video_filename[dot+1:]

        # 查询文件类型是否为视频
        if video_type.lower() not in config.video_types:
            # 不是视频文件，异常退出
            logging.info(u"Not a video file: {}".format(video_filename))
            return -2

        # 算文件 cid
        cid = SubtitleDownloader.cid_hash_file(video)
        print(cid)
        # 获取指定数量的字幕list
        subtitle_list = SubtitleDownloader.fetch_subtitle_list(cid)
        if not subtitle_list:
            logging.info(u"No rSubs available on the server.")
        else:
            logging.info(u"Fetching {} subtitles for: {}".format(len(subtitle_list), video_filename))
        for num, subtitle in enumerate(subtitle_list):
            # 下载字幕
            SubtitleDownloader.download_srt(subtitle, video_base_path, video_name, num)
        logging.info(u"Done.")

    @staticmethod
    def inotify_loop(watch_dir):
        if not isinstance(watch_dir, bytes):
            watch_dir = str.encode(watch_dir)
        import inotify.adapters, inotify.constants
        try:
            i = inotify.adapters.InotifyTree(watch_dir, mask=inotify.constants.IN_DELETE)
            for event in i.event_gen():
                if event is not None:
                    try:
                        (header, type_names, watch_path, filename) = event
                        filename = filename.decode('utf-8')
                        watch_path = watch_path.decode('utf-8')
                        if 'IN_DELETE' in type_names and filename.endswith(u'.aria2'):
                            video_filename = filename[:filename.rfind(u'.')]  # But this maybe a folder!
                            SubtitleDownloader.download_subtitle(os.path.join(watch_path, video_filename))
                    except:
                        pass
        except:
            pass



#!/usr/bin/python
# -*- coding: utf-8 -*-
import os

__author__ = u'rfooox'
__version__ = u'0.0.1'
__email__ = u'rfooox@outlook.com'
video_types = [u'mkv', u'mp4', u'avi', u'rm', u'rmvb', u'wmv', u'webm', u'mpeg', u'mpe', u'flv', u'3gp', u'mov']
subtitle_types = [u'ass', u'srt', u'sub', u'sst', u'son', u'ssa', u'smi', u'tts', u'psb', u'pjs', u'stl', u'vsf']
download_count = 3

http_dns = os.environ.get('SUBTITLE_HTTP_DNS')  # can be one of 'dnspod', 'cf', 'cloudflare', or leave it None
if http_dns not in ('dnspod', 'cf', 'cloudflare'):
    http_dns = None




# -*- coding: utf-8 -*-
import argparse
import os
import logging
from rSubs.subtitle_downloader import SubtitleDownloader


def main():
    logging.basicConfig(level=logging.INFO)
    parser = argparse.ArgumentParser()
    parser.add_argument('-v', '--version', action='version', version=SubtitleDownloader.version(),
                        help=u'show version info'
                        )
    parser.add_argument('-c', '--cid', action='store_true', help=u'show file cid info')
    # parser.add_argument('-d', '--download', action='store', help=u'download subtitles with cid')
    parser.add_argument('-l', '--loop', action='store_true', dest='loop', default=False,
                        help=u'watch a directory recursively, automatically download '
                             u'subtitles when an aria2 task finishes'
                        )
    parser.add_argument('dest', action='store', default='.',
                        help=u'path of directory or video file you need to download subtitles for')

    params = parser.parse_args()
    # 获取文件cid
    if params.cid:
        if os.path.isfile(params.dest):
            temp_cid = SubtitleDownloader.cid_hash_file(params.dest)
            logging.info(f"{params.dest} cid: {temp_cid}")

    # 判断是否为监测文件夹变动
    if params.loop:
        if os.path.isdir(params.dest):
            logging.info(u"Running...")
            # 监测文件夹 执行loop
            SubtitleDownloader.inotify_loop(params.dest)
        else:
            # 不是文件夹 退出监测
            logging.error(u"The given path is invalid.")
    else:
        # 执行 下载任务（参数为文件夹或者文件）
        SubtitleDownloader.download_subtitle(params.dest)
    logging.info(u"Bye :)")


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
import os
import codecs
from setuptools import setup, find_packages
from setuptools.command.install import install
from rSubs.config import __version__, __author__, __email__


ROOT = os.path.abspath(os.path.dirname(__file__))

setup(
    name='rSubs',
    version=__version__,
    author=__author__,
    author_email=__email__,
    keywords='rSubs, subtitles, downloader',
    description='A cli tool to download subtitles.',
    long_description='# A cli tool to download subtitles.\n'
                     'Usage: `sub movie.mkv`, `sub movie_dir`.\n'
                     'Run `sub -h` to see all usages.',
    url='https://github.com/rfooox/rSubs',
    packages=find_packages(),
    package_data={'': ['LICENSES']},
    include_package_data=True,
    zip_safe=False,
    install_requires=['requests>=2.17', 'urllib3>=1.23', 'inotify>=0.2.10'],
    entry_points={
        'console_scripts': [
            'sub = rSubs.main:main',
            'rSubs = rSubs.main:main'
        ]
    },
    license='MIT License',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Other Audience',
        'Natural Language :: Chinese (Traditional)',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy'
    ],
)



import requests
import random
import re
import time
import logging
from urllib3.util import connection


class HTTPDNSBase(object):
    _instance = None

    def __init__(self):
        self.session = requests.session()
        self.whiteList = []
        self.cache = {}

    def __new__(cls, *args, **kw):
        if not cls._instance:
            cls._instance = super(HTTPDNSBase, cls).__new__(cls, *args, **kw)
            cls._instance._initialized = False
        return cls._instance

    def cacheLookup(self, domain):
        standard_domain = self.toStandardDomain(domain)
        if standard_domain not in self.cache:
            return []
        ips, expire = self.cache[standard_domain]
        if time.time() < expire:
            return ips
        return []

    def cacheUpdate(self, domain, ips):
        if not ips:
            return
        standard_domain = self.toStandardDomain(domain)
        self.cache[standard_domain] = (ips, time.time() + 300)

    def toStandardDomain(self, domain):
        domain = domain if domain.endswith('.') else domain + '.'
        return domain

    def resolve(self, domain):
        standard_domain = self.toStandardDomain(domain)
        if re.match(r'^((\d{1,2}|1\d{2}|2[0-4]\d|25[0-5])\.){3}(\d{1,2}|1\d{2}|2[0-4]\d|25[0-5])$', domain):
            return [domain]
        if standard_domain in self.whiteList:
            return [standard_domain]
        return self.cacheLookup(domain)


class HTTPDNSCloudflare(HTTPDNSBase):
    def __init__(self):
        if self._initialized:
            return
        self._initialized = True
        super(HTTPDNSCloudflare, self).__init__()
        self.whiteList = ["cloudflare-dns.com."]

    def resolve(self, domain):
        ret = super(HTTPDNSCloudflare, self).resolve(domain)
        if ret:
            return ret
        domain = self.toStandardDomain(domain)
        url = 'https://cloudflare-dns.com/dns-query?name={}&type=A'
        headers = {'accept': 'application/dns-json'}
        cnames = []
        try:
            res = self.session.get(url.format(domain), headers=headers)
            json_data = res.json()
            for answer in json_data['Answer']:
                if answer['name'] == domain and answer['type'] == 1:
                    ret.append(answer['data'])
                if answer['name'] == domain and answer['type'] == 5:
                    cnames.append(answer['data'])
        except:
            pass
        if ret:
            self.cacheUpdate(domain, ret)
            return ret
        for cname in cnames:
            ret += self.resolve(cname)
        self.cacheUpdate(domain, ret)
        return ret


class HTTPDNSDnspod(HTTPDNSBase):
    def __init__(self):
        if self._initialized:
            return
        self._initialized = True
        super(HTTPDNSDnspod, self).__init__()
        self.whiteList = ["119.29.29.29."]

    def resolve(self, domain):
        ret = super(HTTPDNSDnspod, self).resolve(domain)
        if ret:
            return ret
        url = 'http://119.29.29.29/d?dn={}'
        try:
            res = self.session.get(url.format(domain)).text
            ips = res.split(';')
            ret += ips
        except:
            pass
        self.cacheUpdate(domain, ret)
        return ret


def dns_hijack(http_dns=None):
    httpDNS = HTTPDNSBase
    if http_dns == 'dnspod':
        httpDNS = HTTPDNSDnspod
    elif http_dns in ['cf', 'cloudflare']:
        httpDNS = HTTPDNSCloudflare

    _orig_create_connection = connection.create_connection

    def patched_create_connection(address, *args, **kwargs):
        """Wrap urllib3's create_connection to resolve the name elsewhere"""
        # resolve hostname to an ip address; use your own
        # resolver here, as otherwise the system resolver will be used.
        host, port = address
        hostnames = httpDNS().resolve(host)
        logging.debug("Resolve {}: Got {} results.".format(host, len(hostnames)))
        if hostnames:
            host = random.choice(hostnames)
        return _orig_create_connection((host, port), *args, **kwargs)
    connection.create_connection = patched_create_connection
